/*
 * media-test - Simple utility for testing various memory media
 * Copyright (C) 2004 Ales Hakl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "tester.h"
#include "output.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <inttypes.h>

int main(int argc, char**argv){

  test_context_t ctx;
  output_context_t* out;
  off_t last,cur;
  off_t iterations=0;
  int64_t time;
  int res;
  size_t block = 2048;
  int verbose=0;
  char* rand_source=NULL;
  char* reader_name="unix";

  out = stdout_init();
  unix_fill_context(&ctx);

  while(1){
    switch (getopt(argc,argv,"b:xfdvVh?i:r:")){
    case 'b':
      block = atoi(optarg)*1024;
      break;
    case 'x':
      output_done(out);
      out = xml_init();
      if (!out){
	perror("gnuplot_init()");
	return 3;
      }
      break;
    case 'd':
      unix_fill_context_direct(&ctx);
      reader_name="unix (O_DIRECT)";
      break;
    case 'v':
      ++verbose;
      break;
    case '?':
    case 'h':
      puts("Usage: mediatest [-dv] [-b block-size] [-i count] [-p gnuplot-basename] [-r filename] <device>");
      puts("\nOptions:");
      puts(" -b Set block size");
      puts(" -d Use O_DIRECT");
      puts(" -i Perform random seeks");
      puts(" -r Set source of random data");
      puts(" -v Increase verbosity");
      puts(" -x XML output");
      puts("\nArguments:");
      puts(" device Name of file or device to test (testing files is not much useful).\n");
    case 'V':
      puts("media-test version " VERSION);
      return 0;
    case 'r':
      rand_source=optarg;
      break;
    case 'i':
      iterations=atoll(optarg);
      break;
    case -1:
      goto start;
    }
  }

 start:

  if (argc-optind!=1){
    fputs("Too few arguments\n",stderr);
    return 1;
  }

  if (test_init(&ctx,argv[optind])==T_ERROR){
      perror("open");
      return 2;
    
  }

  if(test_alloc(&ctx,block)==T_ERROR){
      perror("malloc");
      return 2;
  }

  last = test_blast(&ctx);
  if (last==(off_t)-1){
      perror("lseek");
      return 2;
    
  }
  fprintf(stderr,"Reading %llu bytes from %s\n",last,argv[optind]);

  if (!iterations){
    output_sequential_head(out,block,reader_name,argv[optind]);
    for (cur=0;cur < last;cur+=block){
      fprintf(stderr,"%llx - %f%%\r",cur,(double)cur/(double)last*100);
      res=time_bread(&ctx,cur,&time);
      if (res==T_ERROR){
	fputs("Bread terror!\n",stderr);
	break;
      }
      if ((verbose>0) && res==T_FAIL){
	char buf[32];
	snprintf(buf,32,"FAILED: %llx",cur);
	perror(buf);
      }
      output_sequential_block(out,cur,time,res,test_errcnt(&ctx));
      test_ecreset(&ctx);
    }
  }else{
    int urandomfd;
    off_t last_off=0;

    if (!rand_source){
      urandomfd = open("/dev/urandom",O_RDONLY);
    } else {
      urandomfd = open(rand_source,O_RDONLY);
    }

    if (urandomfd<0){
      perror("Cannot open random source");
      test_done(&ctx);
      return 5;
    }

    output_random_head(out,block,iterations,reader_name,argv[optind]);

    for (cur=0;cur < iterations;++cur){
      off_t offset;
      uint64_t buf;
      read(urandomfd,&buf,sizeof(off_t));
      offset=(double)buf/(double)UINT64_MAX * (last);
      offset/=block;
      offset*=block;
      fprintf(stderr,"%llx - %f%%\r",offset,((double)cur+1)/(double)iterations*100);
      res=time_bread(&ctx,offset,&time);
      if (res==T_ERROR){
	fputs("Bread terror!\n",stderr);
	break;
      }
      if ((verbose>0) && res==T_FAIL){
	char buf[32];
	snprintf(buf,32,"FAILED: %llx",cur);
	perror(buf);
      }
      output_random_block(out,offset,time,res,test_errcnt(&ctx),abs(offset-last_off));
      last_off=offset;
      test_ecreset(&ctx);
    }

    close(urandomfd);

  }
  fputs("\nDone.\n",stderr);
  test_done(&ctx);
  output_done(out);
}
